local Vector2 = require("/Scripts/vector2")
local camera = {}
camera.type = "camera"
camera.Position = Vector2:new(0, 0)
camera.dimensions = Vector2:new(love.graphics.getWidth(), love.graphics.getHeight())
camera.scaleX = 1
camera.scaleY = 1
camera.rot = 0
camera.attachedTo = nil
camera.id = 0
camera.currentworld = nil

function camera:attach(entity) self.attachedTo = entity end

function camera:draw()end

function camera:set()
	love.graphics.push()
	love.graphics.rotate(-self.rot)
	love.graphics.scale(1 / self.scaleX, 1 / self.scaleY)
	love.graphics.translate(-self.Position.x, -self.Position.y)
end

function camera:unset()
  love.graphics.pop()
end

function camera:move(dx, dy)
	self.Position.x = self.Position.x + (dx or 0)
	self.Position.y = self.Position.y + (dy or 0)
end

function camera:follow()
	if self.attachedTo then 
		local delta_x = self.Position.x - (self.attachedTo.width / 2 + self.attachedTo.Position.x - self.dimensions.x/2)
		local delta_y = self.Position.y - (self.attachedTo.height / 2 + self.attachedTo.Position.y - self.dimensions.y/2)

		self.Position.x = math.floor(self.Position.x - (delta_x * 0.03))
		self.Position.y = math.floor(self.Position.y - (delta_y * 0.03))
	end
end

function camera:rotate(dr)
	self.rot = self.rot + dr
end

function camera:scale(sx, sy)
	sx = sx or 1
	self.scaleX = self.scaleX * sx
	self.scaleY = self.scaleY * (sy or sx)
end

function camera:setPosition(x, y)
	self.Position.x = x or self.Position.x
	self.Position.y = y or self.Position.y
end

function camera:setScale(sx, sy)
	self.scaleX = sx or self.scaleX
	self.scaleY = sy or self.scaleY
end

function camera:mousePosition()
	return love.mouse.getX() * self.scaleX + self.Position.x, love.mouse.getY() * self.scaleY + self.Position.y
end

function camera:run()
	self:follow()
end

function camera:new(o)
	o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

return camera