local system = {}
local Vector2 = require("/Scripts/vector2")
local grid = require("/Scripts/grid")
local mworld = require("/Scripts/worldlist").mworld
local keyevents = require("/Scripts/keyevents")
system.keysdown = {}

function system:CheckCollision(entity1, entity2)
    if entity1.Position.x < entity2.Position.x+entity2.width and entity2.Position.x < entity1.Position.x+entity1.width and entity1.Position.y < entity2.Position.y+entity2.width and entity2.Position.y < entity1.Position.y+entity1.height then
        return true
    end
    return false
end

function system:getInstancesof(world, type)
    local got = {}
    for i, entity in ipairs(world.entities) do
        if entity.type == type then
            table.insert(got, 1, entity)
        end
    end
    return got
end

function system:add(world, entity, s)
    local o = entity:new()
    if not s then
        o.id = #world.entities+1
        o.currentworld = world
        table.insert(world.entities, #world.entities+1, o)
        return world.entities[#world.entities]
    else
        o.id = #world.special+1
        o.currentworld = world
        table.insert(world.special, #world.special+1, o)
        return world.special[#world.special]
    end
end

function system:getcells(world)
    local blocks = {}

    local function add_entity(entity)
        local cells = grid:get(entity)

        for i, cell in ipairs(cells) do
            if blocks[cell] == nil then
                blocks[cell] = {}
            end
            table.insert(blocks[cell], entity)
        end
    end

    for i, entity in ipairs(world.entities) do
        add_entity(entity)
    end

    for i, entity in ipairs(world.special) do
        if entity.type == "player" then
            add_entity(entity)
        end
    end

    return blocks
end

function system:checkcollisions(world)
    local occupied_cells = self:getcells(world)

    for key, left_entity in ipairs(world.entities) do
        local left_entity_cells = grid:get(left_entity)
        local right_entities = {}
        
        for i, left_entity_cell in ipairs(left_entity_cells) do
            local right_entities = occupied_cells[left_entity_cell]
            
            if right_entities then
                for i2, right_entity in ipairs(right_entities) do
                    
                    if self:CheckCollision(left_entity, right_entity) and left_entity ~= right_entity then
                        if right_entity.Velocity.x > 0 then
                            right_entity.Velocity.x = -right_entity.Velocity.x
                        elseif right_entity.Velocity.x < 0 then
                            right_entity.Velocity.x = math.abs(right_entity.Velocity.x)
                        end
                        if right_entity.Velocity.y > 0 then
                            right_entity.Velocity.y = -right_entity.Velocity.y
                        elseif right_entity.Velocity.y < 0 then
                            right_entity.Velocity.y = math.abs(right_entity.Velocity.y)
                        end
                        right_entities[right_entity.id] = right_entity
                    end
                end
            end
        end
    end
end

function system:draw(world, e)
    world.renderer:render(Vector2:new(math.floor(e.Position.x/32), math.floor(e.Position.y/32)),
    Vector2:new(math.floor((e.Position.x+e.dimensions.x)/32), math.floor((e.Position.y+e.dimensions.y)/32)))
    for i, entity in ipairs(world.entities) do
        entity:draw()
    end
    for i, entity in ipairs(world.special) do
        entity:draw()
    end
end

function system:addkey(keyrl)
    local found = 0
    for i, key in ipairs(self.keysdown) do
        if key == keyrl then
            found = found + 1
        end
    end
    if found == 0 then
        table.insert(self.keysdown, #self.keysdown+1, keyrl)
    end
end

function system:keyreleased(keyrl)
    for i, key in ipairs(self.keysdown) do
        if key == keyrl then
            table.remove(self.keysdown, i)
        end
    end
end

function system:keyrun()
    for i, key in ipairs(self.keysdown) do
        if keyevents.active[key] then keyevents.active[key](mworld.special[2]) end
    end
end

function system:run(world)
    self:checkcollisions(world)
    for i, entity in ipairs(world.entities) do
        entity:run()
    end
    for i, entity in ipairs(world.special) do
        entity:run()
    end
    system:keyrun()
end

return system