local Vector2 = {}

function Vector2:new(ex, ey)
    local v = { x = ex, y = ey }
    setmetatable(v, self)
    self.__index = self
    return v
end

function Vector2:__tostring()
    return "(" .. tostring(self.x) .. ", " .. tostring(self.y) .. ")"
end

function Vector2:__eq(right)
    return self.x == right.x and self.y == right.y
end

function Vector2:floor()
    return Vector2:new(math.floor(self.x), math.floor(self.y))
end

function Vector2:scalar_divide(s)
    return Vector2:new(self.x / s, self.y / s)
end

function Vector2:vector_divide(v)
    return Vector2:new(self.x / v.x, self.y / v.y)
end

function Vector2:scalar_multiply(s)
    return Vector2:new(self.x * s, self.y * s)
end

function Vector2:vector_multiply(v)
    return Vector2:new(self.x * v.x, self.y * v.y)
end

function Vector2:add(v)
    return Vector2:new(self.x + v.x, self.y + v.y)
end

function Vector2:subtract(v)
    return Vector2:new(self.x - v.x, self.y - v.y)
end

function Vector2:length()
    return math.sqrt(self.x * self.x + self.y * self.y)
end

function Vector2:clamp(min, max, v)
    return self:new((v.x < min.x and min.x or (v.x > max.x and max.x or v.x)), (v.y < min.y and min.y or (v.y > max.y and max.y or v.y)))
end

return Vector2
